@@ -9,13 +9,12 @@ from typing import Dict, Set
 from kedro.pipeline.node import Node
 from kfp import Client, dsl
 from kfp.compiler import Compiler
-from kubernetes.client import V1EnvVar
+from kubernetes.client import V1EnvVar, V1SecurityContext
 from tabulate import tabulate
 
+from .auth import IAP_CLIENT_ID, AuthHandler
 from .utils import is_mlflow_enabled
 
-IAP_CLIENT_ID = "IAP_CLIENT_ID"
-
 WAIT_TIMEOUT = 24 * 60 * 60
 
 
@@ -24,13 +23,13 @@ class KubeflowClient(object):
     log = logging.getLogger(__name__)
 
     def __init__(self, config, project_name, context):
-        token = self.obtain_id_token()
-        self.host = config["host"]
+        token = AuthHandler().obtain_id_token()
+        self.host = config.host
         self.client = Client(self.host, existing_token=token)
         self.project_name = project_name
         self.context = context
         dsl.ContainerOp._DISABLE_REUSABLE_COMPONENT_WARNING = True
-        self.volume_meta = config.get("run_config", {}).get("volume")
+        self.volume_meta = config.run_config.volume
 
     def list_pipelines(self):
         pipelines = self.client.list_pipelines(page_size=30).pipelines
@@ -57,42 +56,15 @@ class KubeflowClient(object):
         if wait:
             run.wait_for_run_completion(timeout=WAIT_TIMEOUT)
 
-    def obtain_id_token(self):
-        from google.auth.transport.requests import Request
-        from google.oauth2 import id_token
-        from google.auth.exceptions import DefaultCredentialsError
-
-        client_id = os.environ.get(IAP_CLIENT_ID, None)
-
-        jwt_token = None
-
-        if not client_id:
-            self.log.info(
-                "No IAP_CLIENT_ID provided, skipping custom IAP authentication"
-            )
-            return jwt_token
-
-        try:
-            self.log.debug("Obtaining JWT token for %s." + client_id)
-            jwt_token = id_token.fetch_id_token(Request(), client_id)
-            self.log.info("Obtained JWT token for MLFLOW connectivity.")
-        except DefaultCredentialsError as ex:
-            self.log.warning(
-                str(ex)
-                + (
-                    " Note that this authentication method does not work with default"
-                    " credentials obtained via 'gcloud auth application-default login'"
-                    " command. Refer to documentation on how to configure service account"
-                    " locally"
-                    " (https://cloud.google.com/docs/authentication/production#manually)"
-                )
-            )
-        except Exception as e:
-            self.log.error("Failed to obtain IAP access token. " + str(e))
-        finally:
-            return jwt_token
-
     def generate_pipeline(self, pipeline, image, image_pull_policy):
+        def _customize_op(op):
+            op.container.set_image_pull_policy(image_pull_policy)
+            if self.volume_meta and self.volume_meta.owner is not None:
+                op.container.set_security_context(
+                    V1SecurityContext(run_as_user=self.volume_meta.owner)
+                )
+            return op
+
         @dsl.pipeline(
             name=self.project_name,
             description="Kubeflow pipeline for Kedro project",
@@ -115,29 +87,33 @@ class KubeflowClient(object):
             vop = dsl.VolumeOp(
                 name="data-volume-create",
                 resource_name="data-volume",
-                size=self.volume_meta.get("size", "1Gi"),
-                modes=self.volume_meta.get("access_modes", ["ReadWriteMany"]),
-                storage_class=self.volume_meta.get("storage_class"),
+                size=self.volume_meta.size,
+                modes=self.volume_meta.access_modes,
+                storage_class=self.volume_meta.storageclass,
             )
-            volume_init = dsl.ContainerOp(
-                name="data-volume-init",
-                image=image,
-                command=["sh", "-c"],
-                arguments=[
-                    " ".join(
-                        [
-                            "cp",
-                            "--verbose",
-                            "-r",
-                            "/home/kedro/data/*",
-                            "/home/kedro/datavolume",
-                        ]
+            if self.volume_meta.skip_init:
+                return {"/home/kedro/data": vop.volume}
+            else:
+                volume_init = _customize_op(
+                    dsl.ContainerOp(
+                        name="data-volume-init",
+                        image=image,
+                        command=["sh", "-c"],
+                        arguments=[
+                            " ".join(
+                                [
+                                    "cp",
+                                    "--verbose",
+                                    "-r",
+                                    "/home/kedro/data/*",
+                                    "/home/kedro/datavolume",
+                                ]
+                            )
+                        ],
+                        pvolumes={"/home/kedro/datavolume": vop.volume},
                     )
-                ],
-                pvolumes={"/home/kedro/datavolume": vop.volume},
-            )
-            volume_init.container.set_image_pull_policy(image_pull_policy)
-            return {"/home/kedro/data": volume_init.pvolume}
+                )
+                return {"/home/kedro/data": volume_init.pvolume}
 
         def _build_kfp_ops(
             node_dependencies: Dict[Node, Set[Node]], node_volumes: Dict
@@ -145,46 +121,45 @@ class KubeflowClient(object):
             """Build kfp container graph from Kedro node dependencies. """
             kfp_ops = {}
 
-            env = [
-                V1EnvVar(
-                    name=IAP_CLIENT_ID, value=os.environ.get(IAP_CLIENT_ID, "")
-                )
-            ]
+            iap_env_var = V1EnvVar(
+                name=IAP_CLIENT_ID, value=os.environ.get(IAP_CLIENT_ID, "")
+            )
+            nodes_env = [iap_env_var]
 
             if is_mlflow_enabled():
-                kfp_ops["mlflow-start-run"] = dsl.ContainerOp(
-                    name="mlflow-start-run",
-                    image=image,
-                    command=["kedro"],
-                    arguments=[
-                        "kubeflow",
-                        "mlflow-start",
-                        dsl.RUN_ID_PLACEHOLDER,
-                    ],
-                    file_outputs={"mlflow_run_id": "/tmp/mlflow_run_id"},
+                kfp_ops["mlflow-start-run"] = _customize_op(
+                    dsl.ContainerOp(
+                        name="mlflow-start-run",
+                        image=image,
+                        command=["kedro"],
+                        arguments=[
+                            "kubeflow",
+                            "mlflow-start",
+                            dsl.RUN_ID_PLACEHOLDER,
+                        ],
+                        container_kwargs={"env": [iap_env_var]},
+                        file_outputs={"mlflow_run_id": "/tmp/mlflow_run_id"},
+                    )
                 )
-                kfp_ops["mlflow-start-run"].container.set_image_pull_policy(
-                    image_pull_policy
-                )
-                env.append(
+
+                nodes_env.append(
                     V1EnvVar(
-                        name="MLFLOW_PARENT_ID",
+                        name="MLFLOW_RUN_ID",
                         value=kfp_ops["mlflow-start-run"].output,
                     )
                 )
 
             for node in node_dependencies:
                 name = _clean_name(node.name)
-                kfp_ops[node.name] = dsl.ContainerOp(
-                    name=name,
-                    image=image,
-                    command=["kedro"],
-                    arguments=["run", "--node", node.name],
-                    pvolumes=node_volumes,
-                    container_kwargs={"env": env},
-                )
-                kfp_ops[node.name].container.set_image_pull_policy(
-                    image_pull_policy
+                kfp_ops[node.name] = _customize_op(
+                    dsl.ContainerOp(
+                        name=name,
+                        image=image,
+                        command=["kedro"],
+                        arguments=["run", "--node", node.name],
+                        pvolumes=node_volumes,
+                        container_kwargs={"env": nodes_env},
+                    )
                 )
 
             return kfp_ops
@@ -248,14 +223,17 @@ class KubeflowClient(object):
         with NamedTemporaryFile(suffix=".yaml") as f:
             Compiler().compile(pipeline_func, f.name)
             return self.client.pipeline_uploads.upload_pipeline_version(
-                f.name, name=version_name, pipelineid=pipeline_id
+                f.name,
+                name=version_name,
+                pipelineid=pipeline_id,
+                _request_timeout=10000,
             ).id
 
     def _upload_pipeline(self, pipeline_func, pipeline_name):
         with NamedTemporaryFile(suffix=".yaml") as f:
             Compiler().compile(pipeline_func, f.name)
             pipeline = self.client.pipeline_uploads.upload_pipeline(
-                f.name, name=pipeline_name
+                f.name, name=pipeline_name, _request_timeout=10000
             )
             return (pipeline.id, pipeline.default_version.id)
 
